from jsonschema import Draft7Validator, ValidationError

from pydantic import BaseModel

tags_schema = {"type": "array", "minItems": 1, "items": {"type": "string"}}

schema = {
    "type": "object",
    "required": [
        "__index__",
        "dataset_id",
        "dataset_source",
        "description",
        "publication",
        "kw_data_type",
        "organism",
        "curation_version",
        "total_num_samples",
    ],
    "properties": {
        "__index__": {
            "type": "object",
            "required": ["data_required"],
            "properties": {"data_required": {"enum": ["true", "false"]}},
        },
        "dataset_id": {"type": "string"},
        "dataset_source": {"type": "string"},
        "description": {"type": "string"},
        "publication": {"type": "string"},
        "kw_data_type": {
            "enum": [
                "Transcriptomics",
                "Metabolomics",
                "Mutation",
                "MiRNA",
                "Proteomics",
                "Single cell",
                "Gene dependency",
                "Methylation",
                "Drug Screens",
                "Gene effect",
            ]
        },
        "organism": {"one_of": [tags_schema, {"type": "string"}]},
        "disease": tags_schema,
        "tissue": tags_schema,
        "kw_drug": tags_schema,
        "kw_cell_line": tags_schema,
        "kw_cell_type": tags_schema,
        "curation_version": {"type": "string", "pattern": "^g\d(\.\d)?$"},
        "year": {"type": "integer"},
        "total_num_samples": {"type": "integer", "minimum": 1},
    },
    "if": {
        "properties": {"curation_version": {"enum": ["g2", "g3"]}},
        "required": ["curation_version"],
    },
    "then": {
        "required": [
            "disease",
            "tissue",
            "kw_drug",
            "kw_cell_line",
            "kw_cell_type",
        ]
    },
}

GCT_COLS = [
    "kw_curated_tissue",
    "kw_curated_disease",
    "kw_curated_drug",
    "kw_curated_cell_line",
    "kw_curated_cell_type",
    "kw_curated_genetic_mod_type",
    "kw_curated_modified_gene",
]

Draft7Validator.check_schema(schema)

META_VALIDATOR = Draft7Validator(schema)


class Validator:
    """
    from faircheck import Validator

    # Quick check
    Validator(quilt_metadata).print_errors()

    # Other convenience functions
    validator = Validator(quilt_metadata)
    print("Is valid? ", validator.is_valid())

    # Iterate over errors if needed
    for error in validator.iter_errors():
        print(error.message)
    """

    def __init__(self, quilt_metadata, gctoo=None):
        self.quilt_metadata = quilt_metadata
        self.gctoo = gctoo
        self._grade = None
        if (
                type(quilt_metadata) == dict
                and "curation_version" in quilt_metadata
                and quilt_metadata["curation_version"] == "g3"
        ):
            self._grade = "g3"

    def _iter_gct_errors(self):
        """
        This checks whether gct column metadata has required columns
        for grade 3. Yields errors one by one.
        """
        for col in GCT_COLS:
            if col not in self.gctoo.col_metadata_df.columns:
                yield ValidationError(f"{col} not in gct column metadata")

        if "None" in self.gctoo.col_metadata_df["kw_curated_disease"].values:
            yield ValidationError(
                f"'None' not allowed in 'kw_curated_disease'. Use 'Normal' as default value."
            )

    def is_valid(self):
        """
        Returns True or False depending on validity of data
        """
        return META_VALIDATOR.is_valid(self.quilt_metadata)

    def iter_errors(self):
        """
        This can be used to iterate over all errors.

        for error in validator.iter_errors():
                print(error.message)

        for error in META_VALIDATOR.iter_errors(self.quilt_metadata):
            yield error

        if self._grade == "g3" and self.gctoo:
            for error in self._iter_gct_errors():
                yield error
        """

    def print_errors(self):
        for error in self.iter_errors():
            print(error.message)
